from wenxin_api.api.base_api import BaseObject, APIBaseObject
from wenxin_api.api.base_api import CreatableAPIObject, DeletableAPIObject, ListableAPIObject
from wenxin_api.api.task_api import Task
from wenxin_api.api.model_api import Model
from wenxin_api.api.dataset_api import Dataset
from wenxin_api.api.train_api import Train


# wenxin api python bindings.

import os
from typing import Optional

from wenxin_api.api import Task, Dataset, Train, Model
from wenxin_api.error import WenxinError, APIError, InvalidRequestError
from wenxin_api.requestor import WenxinAPIResponse
from wenxin_api import const, log, version

ak = os.environ.get("WENXINAPI_AK", None)
sk = os.environ.get("WENXINAPI_SK", None)
access_token = os.environ.get("WENXINAPI_ACCESS_TOKEN", None)
debug = False
proxy=None

__all__ = [
    "Task"
    "Dataset",
    "Train",
    "Model",
    "APIError",
    "WenxinError",
    "InvalidRequestError",
    "ak",
    "sk",
    "access_token",
    "api_type",
    "log",
    "const",
    "version",
]


import time

from wenxin_api import util, error
from wenxin_api.api import Task
from wenxin_api.error import TryAgain

import time
import warnings

from wenxin_api import util
from wenxin_api.api import ListableAPIResource, UpdateableAPIResource
from wenxin_api.error import InvalidAPIType, TryAgain
from wenxin_api.util import ApiType
logger = util.get_logger()

class TextGeneration(Task):
    OBJECT_NAME = "text_generation"

    @classmethod
    def create(cls, *args, **kwargs):
        """ text generation task """
        start = time.time()
        timeout = kwargs.pop("timeout", None)
        kwargs["request_id"] = 0

        while True:
            try:
                return super().create(*args, **kwargs)
            except TryAgain as e:
                if timeout is not None and time.time() > start + timeout:
                    raise

                logger.info("Waiting for model to warm up, msg: {}".format(e))


import json
import os
import time
from typing import cast

import wenxin_api
from wenxin_api import requestor, util, error, log
from wenxin_api.api import CreatableAPIObject, ListableAPIObject, DeletableAPIObject
from wenxin_api.error import TimeOutError, NotReady, MissingRequestArgumentError
from wenxin_api.const import CMD_UPLOAD_DATA, CMD_QUERY_DATA, CMD_DELETE_DATA
from wenxin_api.variable import REQUEST_SLEEP_TIME
logger = log.get_logger()


class Dataset(CreatableAPIObject, ListableAPIObject, DeletableAPIObject):
    OBJECT_NAME = "dataset"
    @classmethod
    def create(
        cls,
        local_file_path,
        ak=None,
        sk=None,
        api_type=0,
        **params
    ):
        timeout = params.pop("timeout", None)
        http_requestor = requestor.HTTPRequestor(ak, sk)
        files = {"file": ("file", open(local_file_path, 'rb'))}
        wenxin_response = cls.default_request(ak=ak,
                                              sk=sk,
                                              api_type=api_type, 
                                              request_id=CMD_UPLOAD_DATA, 
                                              files=files)
        dataset_id = wenxin_response.id
        while True:
            try:
                return cls.retrieve(data_id=dataset_id, ak=ak, sk=sk, api_type=api_type)
            except NotReady as e:
                if timeout is not None and time.time() > start + timeout:
                    raise TimeOutError
                logger.info("please wait while the dataset is uploading, msg: {}".format(e))
                time.sleep(REQUEST_SLEEP_TIME)

        return response

    @classmethod
    def retrieve(cls, *args, **params):
        request_id = CMD_QUERY_DATA
        params["type"] = "data"
        api_type = 0
        if "data_id" not in params:
            raise MissingRequestArgumentError("data_id is not provided")
        return super().retrieve(args, api_type=api_type, request_id=CMD_QUERY_DATA, **params)

    @classmethod
    def list(cls, *args, **params):
        request_id = CMD_QUERY_DATA
        params["type"] = "data"
        api_type = 0
        return super().list(args, api_type=api_type, request_id=CMD_QUERY_DATA, **params)

    @classmethod
    def delete(cls, *args, **params):
        request_id = CMD_DELETE_DATA
        params["type"] = "data"
        api_type = 0
        if "data_id" not in params:
            raise MissingRequestArgumentError("data_id is not provided")
        return super().retrieve(args, api_type=api_type, request_id=CMD_DELETE_DATA, **params)




import time

from wenxin_api import requestor, util, error, log
from wenxin_api.api import Task
from wenxin_api.error import TryAgain

import time
import warnings

from wenxin_api import util
from wenxin_api.api import ListableAPIObject
from wenxin_api.variable import REQUEST_SLEEP_TIME
logger = log.get_logger()

class TextToImage(Task):
    OBJECT_NAME = "text_to_image"

    @classmethod
    def create(cls, *args, **params):
        """ text generation task """
        # hard code
        create_url = "https://wenxin.baidu.com/younger/portal/api/rest/1.0/ernievilg/v1/txt2img"
        retrieve_url = "https://wenxin.baidu.com/younger/portal/api/rest/1.0/ernievilg/v1/getImg"
        start = time.time()
        timeout = params.pop("timeout", None)
        text = params.pop("text", "")
        style = params.pop("style", "")
        http_requestor = requestor.HTTPRequestor()
        resp = http_requestor.request(create_url, text=text, style=style, return_raw=True)
        task_id = resp.json()["data"]["taskId"]
        not_ready = True
        while not_ready:
            resp = http_requestor.request(retrieve_url, taskId=task_id, return_raw=True)
            not_ready = resp.json()["data"]["status"] == 0
            if not not_ready:
                return resp.json()
            logger.info("model is painting now!, msg:{}\n".format(resp.json()))
            time.sleep(REQUEST_SLEEP_TIME)


import wenxin_api

def convert_to_wenxin_object(resp, return_raw=False):
    # convert wenxin_api response to wenxin_api object.

    response_ms: Optional[int] = None
    if isinstance(resp, wenxin_api.wenxin_response.WenxinResponse):
        resp = resp.data

    if return_raw:
        return resp
    elif isinstance(resp, list):
        return [convert_to_object(one_rst) for one_rst in resp]
    elif isinstance(resp, dict) and not isinstance(
        resp, wenxin.wenxin_object.WenxinObject
    ):
        resp = resp.copy()
        klass_name = resp.get("object")
        if isinstance(klass_name, str):
            klass = get_object_classes().get(
                klass_name, wenxin.wenxin_object.WenxinObject
            )
        else:
            klass = wenxin.wenxin_object.WenxinObject

        return klass.construct_from(
            resp,
            api_key=api_key,
            api_version=api_version,
            organization=organization,
            response_ms=response_ms,
            engine=engine,
        )
    else:
        return 